;---------------------------------------------
; in interrupt routine
;---------------------------------------------

I_INTERRUPT:
	DI
	PUSH	AF
	IN	A,(0x64)
	PUSH	AF

	IN	A,(0x99)	; CHK OF INT VAN VDP IS 
	AND	A
	JP	P,I_NO_INT_V9938

	; 50/60 INT TROEP 

	PUSH	BC
	PUSH	DE
	PUSH	HL

	CALL	MOUSE
	CALL	I_READ_BUTTONS
	CALL	I_READ_KEYS

	LD a,(#INT_CNT)
	dec a
	JP	NZ,I_CNT

	LD a,(#SEC_CNT)
	inc a
	LD (#SEC_CNT),a
	LD a,#61
I_CNT:
	LD (#INT_CNT),a

	POP	HL
	POP	DE
	POP	BC
I_NO_INT_V9938:
	POP	AF
	OUT	(0x64),A
	POP	AF
	EI
	RETI

I_READ_KEYS:
        IN	A,(0xAA)
        AND	#0xF0
        LD	C,A
        LD	B,#11
        LD	HL,#KEYS_TAB+10
I_READ_KEYS_LP:
        LD	A,C
        DEC	B
        OR	B
        INC	B
        OUT	(0xAA),A
        LD	A,(DE)
        IN	A,(0xA9)
        LD	(HL),A
        DEC	HL
        DJNZ	I_READ_KEYS_LP
	RET

KEYS_TAB:	; BIT :	   7    6    5    4    3    2    1    0   
	.db	0	;  7    6    5    4    3    2    1    0		0
	.db	0	;  ;    }    {    \    =    -    9    8		1
	.db	0	;  B    A    ACC  /    .    ,    '    '		2
	.db	0	;  J    I    H    G    F    E    D    C		3
	.db	0	;  R    Q    P    O    N    M    L    K		4
	.db	0	;  Z    Y    X    W    V    U    T    S		5
	.db	0	;  F3   F2   F1   CODE CAPS GRFH CTRL SHIFT	6 NZ
	.db	0	;  RET  SEL  BS   STOP TAB  ESC  F5   F4	7 Z
	.db	0	;  RI   DO   UP   LE   DEL  INS  HOME SPACE	8 Z

	.db	0	;  4    3    2    1    0    /    +    *		9
	.db	0	;  .    ,    -    9    8    7    6    5		10


I_READ_BUTTONS:
	XOR	A
	CALL	I_FIR_P1
	LD	(FIRE_BUTTONS),A
	RET


I_FIR_P1:
	LD	B,A	; A=0 PORT 1 / A=1 PORT 2 
	LD	A,#15

	OUT	(0xA0),A
	NOP
	IN	A,(0xA2)

	DJNZ	I_FIR_S1
	AND	#0xDF
	OR	#0x4C
	JR	I_FIR_S2
I_FIR_S1:	AND	#0xAF
	OR	#3
I_FIR_S2:	OUT	(0xA1),A
	LD	A,#14
	OUT	(0xA0),A
	NOP
	IN	A,(0xA2)
	RET

MOUSE:
	; Gebruik spratr om x en y op te slaan ! 

	LD	A,(MOUSE_PORT)	; Get mouse port in C 
	LD	C,A
MOUSE1:
	LD	A,(MOUSE_WAIT1)
	LD	B,A	; Get Y move 
	CALL	READ_MOUSE
	RLCA
	RLCA
	RLCA
	RLCA
	LD	H,A
	CALL	READ_MOUSE
	OR	H
	LD	H,A
	CALL	READ_MOUSE	; Get X move 
	RLCA
	RLCA
	RLCA
	RLCA
	LD	L,A
	CALL	READ_MOUSE
	OR	L
	LD	C,H	; B = X-move / C = Y-move 
	LD	B,A

	LD	A,B	; Move -1,-1 ? 
	INC	A
	LD	H,A
	LD	A,C
	INC	A
	OR	H
	JR	NZ,MOUSE3

	LD	A,(MOUSE_USE)	; Check mouse use 
	OR	A
	JR	NZ,MOUSE2

	LD	A,(MOUSE_PORT)	; Check other port 
	XOR	#0b01000000
	LD	(MOUSE_PORT),A

	JR	KEYBOARD

MOUSE2:
	LD	A,(MOUSE_OFF)	; na 20 keer -1,-1 is uit 
	INC	A
	LD	(MOUSE_OFF),A
	CP	#20
	JR	C,MOUSE4
	XOR	A	; Deactivate mouse 
	LD	(MOUSE_USE),A

	JR	KEYBOARD

MOUSE3:	; Activate mouse / Keep mouse activat 
	XOR	A	; counter op 0 
	LD	(MOUSE_OFF),A
	DEC	A	; wordt weer gebruikt 
	LD	(MOUSE_USE),A

MOUSE4:
	LD	A,B	; If mouse is moved then skip keyboar 
	OR	C
	JR	NZ,CHK_PIJL_BORD	; niet 0,0 dan check 


KEYBOARD:
	; nog even geen keyboard 
	RET


CHK_PIJL_BORD:	; In: B=y-coordinaat 

	LD	HL,#MOUSEY
	LD	A,(HL)	; Y
	SUB	B

	BIT	7,B
	JR	NZ,CHK_PIJL_BORD_Y1
	JR	NC,CHK_PIJL_BORD_Y3
	XOR	A
	JR	CHK_PIJL_BORD_Y3

CHK_PIJL_BORD_Y1:
	JR	NC,CHK_PIJL_BORD_Y2
	CP	#204
	JR	C,CHK_PIJL_BORD_Y3

CHK_PIJL_BORD_Y2:
	LD	A,#203

CHK_PIJL_BORD_Y3:
	LD	(HL),A	; Y
	INC	HL

	LD	A,(HL)	; X
	SUB	C
	BIT	7,C
	JR	NZ,CHK_PIJL_BORD_X1
	JR	NC,CHK_PIJL_BORD_X3
	XOR	A
	JR	CHK_PIJL_BORD_X3

CHK_PIJL_BORD_X1:
	JR	NC,CHK_PIJL_BORD_X2
	CP	#249
	JR	C,CHK_PIJL_BORD_X3
CHK_PIJL_BORD_X2:
	LD	A,#252
CHK_PIJL_BORD_X3:
	LD	(HL),A	; X
	LD	B,A
MOUSE_SPRITE:
	IN	A,(0x65)
	RRCA
	JR	C,MOUSE_SPRITE	; Command ready

	XOR	A
	OUT	(0x64),A
	OUT	(0x63),A
	LD	A,#0xFE
	OUT	(0x63),A
	LD	A,#0x07
	OUT	(0x63),A
	DEC	HL
	LD	A,(HL)
	OUT	(0x60),A	; Y  
	XOR	A
	OUT	(0x60),A
	OUT	(0x60),A	;BITJE 0=Y8  
	OUT	(0x60),A
	LD	A,B
	SLA	A
	OUT	(0x60),A	; X  
	LD	A,#0
	OUT	(0x60),A
	RL	A
	OR	#0x40
	OUT	(0x60),A	; X

	RET


READ_MOUSE:
	LD	A,#15
	OUT	(0xA0),A
	LD	A,C
	OUT	(0xA1),A
	XOR	#0b00110000
	LD	C,A
	LD	A,#14
	OUT	(0xA0),A
READ_MOUSE_L:
	DJNZ	READ_MOUSE_L

	LD	A,(MOUSE_WAIT2)
	LD	B,A
	IN	A,(0xA2)
	AND	#0x0F
	RET
